% This is the main run file for producing the forecasting results in 
% Chan and Eisenstat (2017).
%
% This code is free to use for academic purposes only, provided that the 
% paper is cited as:
%
% Chan, J.C.C. and Eisenstat, E. (2017). Efficient Estimation of Bayesian
% VARMAs with Time-Varying Coefficients, Journal of Applied Econometrics, 
% forthcoming 
%
% This code comes without technical support of any kind. It is expected to
% reproduce the results reported in the paper. Under no circumstances will
% the authors be held responsible for any use (or misuse) of this code in
% any way.

clear; clc;
% 1: RW; 2: VAR; 3: VAR-SV; 4: VARMA; 5: VARMA-SV1; 6: VARMA-SV2; 7: TVP-VARMA-SV2; 
model = 7;
nloop = 25000;
burnin = 5000;
p = 2; % lag length, p <= 4

% load data
scale_data = 0;  % standardize all series to have std. dev. of 1
center_data = 0; % standardize all series to have mean 0
Ydata = load( 'data_Q.csv' ); % 1959Q3 - 2013Q2
Ydata = Ydata( :, [2 1] ); % n = 2; CPI, GDP
[T, n] = size( Ydata );

% standardize the data if so desired
dscale = 1 + scale_data .* ( std( Ydata, 1 ) - 1 );
dcenter = center_data .* mean( Ydata, 1 );
Ydata = ( Ydata  - repmat( dcenter, T, 1 ) ) ./ repmat( dscale, T, 1 );

Y0 = Ydata( 1:p, : );
shortY = Ydata( p+1:end, : );
T = T - p;
Y = reshape(shortY',T*n,1);
k = n+p*n^2;    
kb = n+p*n^2;           % number of AR coefficients
    
% recursive forecast exercise
T0 = 59-p; %% evaluation period -- start from 1974Q1

switch model
    case 1 
        RW;        
    case 2        
        VAR;
    case 3
        VAR_SV;
    case 4
        VARMA;
    case 5
        VARMA_SV1;
    case 6
        VARMA_SV2;
    case 7        
        VARMA_TVP_SV2;
end

results = zeros(3,3);
results(1,:) = sum(yhat1(4:end,:));
results(2,:) = sum(yhat2(3:end,:));
results(3,:) = sum(yhat3(2:end,:));

clc;
fprintf(['Sum of log predictive likelihoods for ' model_name ':'])
fprintf('\n'); 
fprintf('                 | inflation, GDP growth, joint\n'); 
fprintf('one-step-ahead   | %.1f, %.1f, %.1f\n', results(1,1), results(1,2), results(1,3)); 
fprintf('two-step-ahead   | %.1f, %.1f, %.1f\n', results(2,1), results(2,2), results(2,3)); 
fprintf('three-step-ahead | %.1f, %.1f, %.1f\n', results(3,1), results(3,2), results(3,3)); 

